<?php
/** @file
 *
 * @note Copyright (C) 2007-2010 Thomas ZILLIOX
 * 
 * This file is part of ZPHOTOS EXPRESS.
 * 
 * ZPHOTOS EXPRESS is free script: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License.
 * 
 * ZPHOTOS EXPRESS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with ZPHOTOS EXPRESS. If not, see <http://www.gnu.org/licenses/>.
 * 
 * You can contact the author by e-mail thomas@zilliox.me
 * 
 */
$sourcePath = dirname(dirname(__FILE__));
require($sourcePath.'/Utils/Zphotos_Utils.php');
require($sourcePath.'/Model/Zphotos_Data.php');
require($sourcePath.'/Model/Zphotos_Album.php');
require($sourcePath.'/Model/Zphotos_Photo.php');
require($sourcePath.'/View/Zphotos_View.php');

/** @class
 * @author Thomas ZILLIOX
 * @date 2007-2010
 * @version 2.0
 */
class Zphotos_Controller {
	
	
	/**************************************************************************
				ATTRIBUTES						
	**************************************************************************/

	/** 
	 * l'album racine. (objet Zphotos_Album)
	 * @private
	 */
	var $rootAlbum;

	/** 
	 * l'album courant. (objet Zphotos_Album)
	 * @private
	 */
	var $currentAlbum;

	/** 
	 * la photo courante. (objet Zphotos_Photo)
	 * @private
	 */
	var $currentPhoto;

	/** 
	 * la page courante. (objet Zphotos_Photo)
	 * @private
	 */
	var $currentPage;

	/** 
	 * les options de Zphotos Express. (objet Zphotos_Options)
	 * @private
	 */
	var $options;
	
	
	/**************************************************************************
				PUBLIC METHODS						
	**************************************************************************/
	/** @name Public methods
	 */
	//@{

	/** 
	 * @public
	 * @param $themeName, Le nom du thème à utiliser
	 * @return Le code HTML de la page à afficher
	 */
	function show() {
		$themeClassName = 'Zphotos_View';
		
		// Vérification du theme
		if (func_num_args() > 0) {
			$themeName = func_get_arg(0);
			$verifyName = str_replace(' ', '_', ucwords(strtolower(str_replace('_', ' ', $themeName))));
			if ($verifyName != $themeName) {
				echo 'Bad name for your theme. "'.$themeName.'" instead of "'.$verifyName.'".';
			} else if (file_exists($this->options->getThemesPath().$themeName)) {
				include($this->options->getThemesPath().$themeName.'/View.php');
				$themeClassName = $themeName.'_View';
			}
		}
		$view = new $themeClassName($this->rootAlbum);
		
		// Récupération du code 
		if ($this->currentPhoto != null) {
			$code = $view->showPhoto($this->currentPhoto);
		} else {
			$code = $view->showAlbum($this->currentAlbum, $this->currentPage);
		}
		return $code;
	}
	//@}
	
	
	/**************************************************************************
				CONSTRUCTOR							
	**************************************************************************/
	/** @name Constructor
	 */
	//@{

	/** 
	 * Constructeur PHP4.
	 * @private (for the visibility in the documentation)
	 * @param $options, les options de Zphotos Express. (Objet Zphotos_Options)
	 * @see __construct()
	 */
	function Zphotos_Controller(&$options) {
		$this->__construct($options);
	}

	/** 
	 * Constructeur.
	 * @public
	 * @param $options, les options de Zphotos Express. (Objet Zphotos_Options)
	 */
	function __construct(&$options) {
		$this->options = &$options;
		$album = null;
		$this->rootAlbum = new Zphotos_Album($options->getPhotosRootPath(), $album, $options);
		
		// Récupération de l'album courant
		if (isset($_GET[$options->getAlbumVarName()])) {
			$currentAlbumPath = $_GET[$options->getAlbumVarName()];
			// Test if the visitor try to see a forbiden fodler (security)
			$tmpPath = eregi_replace('/', '', $currentAlbumPath);
			if (!ereg('\.\.', $tmpPath) && $tmpPath != '') {
				$currentAlbum = $this->rootAlbum->searchAlbum($options->getPhotosRootPath().$currentAlbumPath);
				if ($currentAlbum != null) {
					$this->currentAlbum = $currentAlbum;
				}
			}
		}
		if ($this->currentAlbum == null) {
			$this->currentAlbum = $this->rootAlbum;
		}
		
		// Récupération de la photo courante
		if (isset($_GET[$options->getPhotoVarName()])) {
			$currentPhotoName = $_GET[$options->getPhotoVarName()];
			$this->currentPhoto = $this->currentAlbum->searchPhoto($currentPhotoName);
			
		// Récupération de la page courante
		} else if (isset($_GET[$options->getPageVarName()])) {
			$currentPage = $_GET[$options->getPageVarName()];
			if (is_numeric($currentPage)) {
				$this->currentPage = $currentPage;
			}
		}
	}
	//@}
}
?>