<?php
/** @file
 *
 * @note Copyright (C) 2007-2010 Thomas ZILLIOX
 * 
 * This file is part of ZPHOTOS EXPRESS.
 * 
 * ZPHOTOS EXPRESS is free script: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License.
 * 
 * ZPHOTOS EXPRESS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with ZPHOTOS EXPRESS. If not, see <http://www.gnu.org/licenses/>.
 * 
 * You can contact the author by e-mail thomas@zilliox.me
 * 
 */

/** @class
 * @author Thomas ZILLIOX
 * @date 2007-2010
 * @version 2.0
 */
class Zphotos_Album extends Zphotos_Data {
	
	
	/**************************************************************************
				ATTRIBUTES						
	/*************************************************************************/

	/** 
	 * Liste des photos.
	 * @private
	 * @see _isPhoto()
	 * @see getPhotos()
	 */
	var $photos = array();

	/** 
	 * Liste des sous-albums.
	 * @private
	 * @see getAlbums()
	 */
	var $albums = array();

	/** 
	 * Détermine le numéro de la page courante. (number)
	 * @private
	 */
	var $page = 1;

	/** 
	 * Détermine le nombre de photos à afficher par pages. (number)
	 * Si cette propriété a pour valeur 0, on affiche toutes les photos sur une seule page.
	 * @private
	 */
	var $paging = 0;
	
	
	/**************************************************************************
				ACCESSORS						
	**************************************************************************/
	/** @name Accessors
	 */
	//@{

	/** 
	 * @public
	 * @param le numero de la page. (nombre) (facultatif)
	 * @return la liste des photos de la page de l'album. (tableau d'objets Photo)
	 * @see getPhotosNumber()
	 * @see getFirstPhoto()
	 * @see getRandomPhoto()
	 */
	function getPhotos() {
		if (func_num_args() > 0) {
			$page = func_get_arg(0);
			if (is_numeric($page) && $this->getPhotosNumberByPage() > 0) {
				$photos = array_slice($photos, ($page-1)*$this->getPhotosNumberByPage(), $this->getPhotosNumberByPage());
			}
		}
		
		return $this->photos;
	}

	/** 
	 * @note Cette méthode peut être utile pour présenter l'album.
	 * @public
	 * @return la première photo de l'album, ou s'il est vide, d'un des sous-albums. (objet Photo)
	 * @see getRandomPhoto()
	 */
	function getFirstPhoto() {
		if (count($this->photos) > 0) {
			return $this->photos[0];
		}
		if (count($this->albums) > 0) {
			return $this->albums[0]->getFirstPhoto();
		}
	}

	/** 
	 * @note Cette méthode peut être utile pour présenter l'album.
	 * @public
	 * @return une photo aléatoire parmi l'album, ou s'il est vide, des sous-albums. (objet Photo)
	 * @see getFirstPhoto()
	 */
	function getRandomPhoto() {
		if (count($this->photos) > 0) return $this->photos[mt_rand(0, count($this->photos)-1)];
		if (count($this->albums) > 0) return $this->albums[mt_rand(0, count($this->albums)-1)]->getRandomPhoto();
	}

	/** 
	 * @public
	 * @return le nombre de photos présentes dans l'album. (nombre)
	 * @see getPhotos()
	 */
	function getPhotosNumber() {
		return count($this->photos);
	}

	/** 
	 * @public
	 * @return la liste des sous-albums. (tableau d'objet Album)
	 * @see getAlbumsNumber()
	 */
	function getAlbums() {
		return $this->albums;
	}

	/** 
	 * @public
	 * @return le nombre de sous-albums. (nombre)
	 * @see getAlbums()
	 */
	function getAlbumsNumber() {
		return count($this->albums);
	}

	/** 
	 * @public
	 * @return vrai si cet album est un parent de l'album courant. (booleen)
	 */
	function hasAlbums() {
		return ($this->getAlbumsNumber() != 0);
	}

	/** 
	 * @public
	 * @return vrai si cet album est l'album racine. (booleen)
	 */
	function isRootAlbum() {
		return ($this->relativePath == '');
	}

	/** 
	 * @public
	 * @return l'url de l'album. (chaîne de caractères)
	 * @see getBaseUrl()
	 */
	function getUrl() {
		if ($this->isRootAlbum()) return $this->options->getZphotosUrl();
		return $this->getBaseUrl().$this->options->getAlbumExtension();
	}

	/** 
	 * @note Cette url a pour but d'être  completée par un sous-album, une photo, etc.
	 * @public
	 * @return l'url de base de l'album. Elle 
	 */
	function getBaseUrl() {
		if ($this->options->isRewriteRule()) {
			return $this->options->getZphotosFolderUrl().zp_rawurlencode($this->relativePath);
		} else {
			return $this->options->getZphotosUrl().'?'.$this->options->getAlbumVarName().'='.zp_rawurlencode($this->relativePath);
		}
	}

	/** 
	 * Est-ce que l'album est vide, c'est-a-dire ne contient aucune photo ?
	 * @public
	 * @param est-ce que vous voulez savoir si mes sous-albums sont vides également ? (booleen) (facultatif, faux par defaut)
	 * @return vrai si l'album est vide. (booleen)
	 */
	function isEmpty() {
		if (func_num_args() > 0) $childs = func_get_arg(0);
		else $childs = false;
		if ($this->getPhotosNumber() > 0) return false;
		if ($childs) {
			foreach($this->albums as $album) {
				if(!$album->isEmpty(true)) return false;
			}
		}
		return true;
	}
	//@}
		
	
	/**************************************************************************
				SORT METHODS						
	**************************************************************************/
	/** @name Sort methods
	 */
	//@{

	/** 
	 * Trie les sous-albums
	 * @public
	 * @param le critère, peut prendre comme valeur 'name' ou 'date'. (chaîne de caractères) (facultatif, 'name', par defaut)
	 * @param voulez-vous trier a l'envers ? (booleen) (facultatif, faux par defaut)
	 * @param dois-je trier mes sous-albums egalement ? (booleen) (facultatif, faux par defaut)
	 * @see sortPhotos()
	 */
	function sortAlbums() {
		if (func_num_args() > 0) $criteria = func_get_arg(0);
		else $criteria = 'name';
		if (func_num_args() > 1) $reverse = func_get_arg(1);
		else $reverse = false;
		if (func_num_args() > 2) $childs = func_get_arg(2);
		else $childs = false;
		
		if (count($this->albums) > 0 && ($criteria == 'name' || $criteria == 'date')) {
			// Do sort
			$albums = array();
			foreach($this->albums as $album) {
				if ($childs) $album->sortAlbums($criteria, $reverse, $childs);
				if ($criteria == 'name') $albums[$album->name] = $album;
				if ($criteria == 'date') $albums[$album->date] = $album;
			}
			if ($reverse) 	krsort($albums);
			else			ksort($albums);
			$this->albums = array_values($albums);			
		}
		return $this;
	}

	/** 
	 * Trie les photos
	 * @public
	 * @param le critère, peut prendre comme valeur 'name' ou 'date'. (chaîne de caractères) (facultatif, 'name', par defaut)
	 * @param voulez-vous trier a l'envers ? (booleen) (facultatif, faux par defaut)
	 * @param dois-je trier les photos de mes sous-albums egalement ? (booleen) (facultatif, faux par defaut)
	 * @see sortAlbums()
	 */
	function sortPhotos() {
		if (func_num_args() > 0) $criteria = func_get_arg(0);
		else $criteria = 'name';
		if (func_num_args() > 1) $reverse = func_get_arg(1);
		else $reverse = false;
		if (func_num_args() > 2) $childs = func_get_arg(2);
		else $childs = false;
		
		if (!$this->isEmpty() && ($criteria == 'name' || $criteria == 'date')) {
			// Do sort
			$photos = array();
			foreach($this->photos as $photo) {
				if ($criteria == 'name') $photos[$photo->name] = $photo;
				if ($criteria == 'date') $photos[$photo->date] = $photo;
			}
			if ($reverse) 	krsort($photos);
			else			ksort($photos);
			$this->photos = array_values($photos);			
		}
		
		if ($childs) {
			$albums = array();
			foreach($this->albums as $album) {
				$albums[] = $album->sortPhotos($criteria, $reverse, $childs);
			}
			$this->albums = $albums;
		}
		return $this;
	}
	//@}
	
	
	/**************************************************************************
				PAGING METHODS					
	**************************************************************************/
	/** @name Pagging methods
	 */
	//@{
	
	/** 
	 * Change le nombre de photo à afficher par page. 
	 * @note Si vous ne voulez pas de système de page, il faut le mettre à 0.
	 * @public
	 * @param nombre maximum de photos a afficher par page. (nombre)
	 */
	function setPhotosNumberByPage($maxSize) {
		if (is_int($maxSize)) $this->paging = $maxSize;
		else $this->paging = 0;
	}
	
	/** 
	 * @public
	 * @return le nombre de photos à afficher par page, 0 si ce n'est pas limité. (nombre)
	 */
	function getPhotosNumberByPage() {
		return $this->paging;
	}

	/** 
	 * @public
	 * @return vrai s'il y a plusieurs pages de photos. (booléen)
	 * @see getPagesNumber()
	 */
	function hasSeveralPages() {
		return ($this->getPagesNumber() > 1);
	}

	/** 
	 * @public
	 * @return le nombre de pages de photo dans l'album. (booléen)
	 * @see setPhotosNumberByPage()
	 */
	function getPagesNumber() {
		if($this->paging > 0) return ceil($this->getPhotosNumber()/$this->paging);
		return 1;
	}

	/** 
	 * @public
	 * @return vrai si la page courante a une page suivante. (booléen)
	 * @see getNextPageUrl()
	 */
	function hasNextPage() {
		if($this->paging > 0) return ($this->page < $this->getPagesNumber());
		return false;
	}

	/** 
	 * @public
	 * @return l'url de la page suivante. (chaîne de caractères)
	 * @see hasNextPage()
	 * @see getPageUrl()
	 * @todo supprimer les référence à $this->page
	 */
	function getNextPageUrl() {
		if ($this->hasNextPage()) {
			return $this->getPageUrl($this->page + 1);
		}
		return null;
	}

	/** 
	 * @public
	 * @return vrai si la page courante a une page precedente. (booléen)
	 * @see getPreviousPageUrl()
	 */
	function hasPreviousPage() {
		return ($this->paging > 0 && $this->page > 1);
	}

	/** 
	 * @public
	 * @return l'url de la page precedente. (chaîne de caractères)
	 * @see hasPreviousPage()
	 * @see getPageUrl()
	 */
	function getPreviousPageUrl() {
		if ($this->hasPreviousPage()) {
			return $this->getPageUrl($this->page - 1);
		}
		return null;
	}

	/** 
	 * @public
	 * @param la photo dont on cherche la page. (objet Photo)
	 * @return l'url de la page contenant d'une photo donnée. (chaîne de caractères)
	 * @see getPageUrl()
	 */
	function getPageUrlForPhoto($photo) {
		$pos = $this->_getPosPhoto($photo->name);
		return $this->getPageUrl(ceil(($pos+1)/$this->paging));
	}

	/** 
	 * @public
	 * @param numero de la page. (nombre) (facultatif, page courante par defaut)
	 * @return l'url de la page recherchée. (chaîne de caractères)
	 */
	function getPageUrl() {
		if (func_num_args() > 0) $page = func_get_arg(0);
		else $page = 0;
		if ($page < 2) return $this->getUrl();
		if ($this->options->isRewriteRule()) {
			return $this->getUrl().$page.$this->options->getPageExtension();
		}
		return $this->getUrl().'&'.$this->options->getPageVarName().'='.$page.$this->options->getPageExtension();
	}
	
	
	//@}
	/**************************************************************************
				PHOTO METHODS					
	**************************************************************************/
	/** @name Photo methods
	 */
	//@{
	
	/** 
	 * @public
	 * @param nom de la photo recherchée. (chaîne de caractères)
	 * @return la photo qui correcpond au nom passé en paramètre. (objet Photo)
	 */
	function searchPhoto($name) {
		$pos = $this->_getPosPhoto($name);
		if ($pos !== null) {
			return $this->photos[$pos];
		}
		return null;
	}

	/** 
	 * Est-ce qu'une photo donnée a une photo suivante ?
	 * @public
	 * @param photo en question. (objet Photo)
	 * @return vrai si la photo passée en parametre a une photo suivante. (booleen)
	 * @see getNextPhoto()
	 */
	function hasNextPhoto($photo) {
		return ($this->getNextPhoto($photo) != null);
	}

	/** 
	 * @public
	 * @param photo en question. (objet Photo)
	 * @return la photo suivante de la photo passée en paramètre. (objet Photo)
	 * @see hasNextPhoto()
	 */
	function getNextPhoto($photo) {
		$pos = $this->_getPosPhoto($photo->name);
		if ($pos !== null && $pos < $this->getPhotosNumber() - 1) {
			return $this->photos[$pos + 1];
		}
		return null;
	}

	/** 
	 * Est-ce qu'une photo donnée a une photo précédente ?
	 * @public
	 * @param photo en question. (objet Photo)
	 * @return vrai si la photo passée en parametre a une photo précédente. (booleen)
	 * @see getPreviousPhoto()
	 */
	function hasPreviousPhoto($photo) {
		return ($this->getPreviousPhoto($photo) != null);
	}

	/** 
	 * @public
	 * @param photo en question. (objet Photo)
	 * @return la photo précédente de la photo passée en paramètre. (objet Photo)
	 * @see hasPreviousPhoto()
	 */
	function getPreviousPhoto($photo) {
		$pos = $this->_getPosPhoto($photo->name);
		if ($pos !== null && $pos > 0) {
			return $this->photos[$pos - 1];
		}
		return null;
	}
	//@}
	
	
	/**************************************************************************
				PUBLIC METHODS							
	**************************************************************************/
	/** @name Public methods
	 */
	//@{

	/** 
	 * @public
	 * @param $path, le chemin correspondant à l'album recherché. (chaîne de caratères)
	 * @return l'album qui correspond au chemin passé en paramètre. (objet Album)
	 */
	function searchAlbum($path) {
		if (!zp_endswith($path,'/')) $path = $path.'/';
		if ($path == $this->path) {
			return $this;
		} else if (zp_istartswith($path, $this->path)) {
			foreach($this->albums as $album) {
				$current = $album->searchAlbum($path);
				if ($current != null) return $current;
			}
		}
	}
	//@}
	
	
	/**************************************************************************
				CONSTRUCTOR							
	**************************************************************************/
	/** @name Constructor
	 */
	//@{

	/** 
	 * Constructeur PHP4.
	 * @private (for the visibility in the documentation)
	 * @param $path, le chemin vers le dossier de l'album. (chaîne de caractères)
	 * @param $album, l'album parent. (objet Zphotos_Album)
	 * @param $options, les options de Zphotos Express. (objet Zphotos_Options)
	 * @see __construct()
	 */
	function Zphotos_Album($path, &$album, &$options) {
		$this->__construct($path, $album, $options);
	}

	/** 
	 * Constructeur.
	 * @public
	 * @param $path, le chemin vers le dossier de l'album. (chaîne de caractères)
	 * @param $album, l'album parent. (objet Zphotos_Album)
	 * @param $options, les options de Zphotos Express. (objet Zphotos_Options)
	 */
	function __construct($path, &$album, &$options) {
		if (!zp_endswith($path,'/')) $path = $path.'/';
		parent::__construct($path, $album, $options);
		
		if ($this->isRootAlbum() && $options->getRootAlbumName() != '') {
			$this->name = $options->getRootAlbumName();
		}
		
		$infos = $this->_parse();
		
		// We parse the subfolders to see if there are subalbums
		foreach ($infos[0] as $folderName) {
			$subalbum = new Zphotos_Album($this->path.$folderName, $this, $options);
			if (!$subalbum->isEmpty(true)) $this->albums[] = &$subalbum;
			unset($subalbum);
		}
		
		// We parse the folder to find the photos
		foreach ($infos[1] as $fileName) {
			if($this->_isPhoto($this->path.$fileName)) {
				$this->photos[] = new Zphotos_Photo($this->path.$fileName, $this, $options);
			}
		}
	}
	//@}
	
	
	/**************************************************************************
				PRIVATE								
	**************************************************************************/
	/** @name Private methods
	 */
	//@{
	
	/**  
	 * @private
	 * @param le fichier. (chaîne de caractères)
	 * @return si le fichier est une photo ou non à partir de son nom. (booléen)
	 */
	function _isPhoto($file) {
		$ext = strtolower( substr( $file, strrpos($file, '.') + 1));
		$extPhoto = array( 'jpeg', 'jpg', 'png', 'gif', 'bmp');
		return ( array_search( $ext, $extPhoto) !== false);
	}
	
	/** 
	 * Récupère la position d'une photo à partir de son nom.
	 * @private
	 * @param le nom de la photo recherchée. (chaîne de caractères)
	 * @return la position de la photo dans l'album. (nombre)
	 */
	function _getPosPhoto($name) {
		$i = 0;
		foreach ($this->photos as $photo) {
			if ($photo->getName() == $name) return $i; 
			$i++;
		}
		return null;
	}

	/** 
	 * @private
	 * @return les sous-dossiers et les fichiers présents dans l'album. (tableau)
	 */
	function _parse() {
		$folders = array();
		$files = array();
		$pointer = opendir($this->path);
		while($file = readdir($pointer)) {
			
			if(is_dir($this->path.$file) && !zp_startswith($file, '_') && ($file != '.') && ($file != '..')) {
				$folders[] = $file;
			}
			if(is_file($this->path.$file)) {
				$files[] = $file;
			}
		}
		closedir($pointer);
		return array($folders, $files);
	}
	//@}
}
?>