<?php
/** @file
 *
 * @note Copyright (C) 2007-2010 Thomas ZILLIOX
 * 
 * This file is part of ZPHOTOS EXPRESS.
 * 
 * ZPHOTOS EXPRESS is free script: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License.
 * 
 * ZPHOTOS EXPRESS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with ZPHOTOS EXPRESS. If not, see <http://www.gnu.org/licenses/>.
 * 
 * You can contact the author by e-mail thomas@zilliox.me
 * 
 */

/** @class
 * @author Thomas ZILLIOX
 * @date 2007-2010
 * @version 2.0
 */
class Zphotos_Data {
	
	
	/**************************************************************************
				ATTRIBUTES						
	/*************************************************************************/

	/** 
	 * L'album parent. (objet Zphotos_Album)
	 * @private
	 * @see getParentAlbum()
	 */
	var $album;

	/** 
	 * Nom du fichier. (chaîne de caractères)
	 * @private
	 * @see getName()
	 * @see setName()
	 */
	var $name = '';

	/** 
	 * Chemin du fichier. (chaîne de caractères)
	 * @private
	 * @see getPath()
	 */
	var $path;

	/** 
	 * Date du fichier. (time)
	 * @private
	 * @see getDate()
	 */
	var $date;

	/** 
	 * Chemin par rapport à l'album racine. (chaîne de caractères)
	 * @private
	 */
	var $relativePath;

	/** 
	 * Description du fichier. (chaîne de caractères)
	 * @private
	 * @see getDescription()
	 * @see getDescriptionHTML()
	 */
	var $description = '';

	/** 
	 * les options de Zphotos Express. (objet Zphotos_Options)
	 * @private
	 */
	var $options;
	
	
	/**************************************************************************
				ACCESSORS						
	**************************************************************************/
	/** @name Accessors
	 */
	//@{

	/** 
	 * @public
	 * @return l'album parent. (objet Zphotos_Album)
	 */
	function hasAlbum() {
		return ($this->album != null);
	}

	/** 
	 * @public
	 * @return l'album parent. (objet Zphotos_Album)
	 */
	function getAlbum() {
		return $this->album;
	}

	/** 
	 * @public
	 * @return Nom du fichier. (chaîne de caractères)
	 * @see setName
	 */
	function getName() {
		return $this->name;
	}

	/** 
	 * @public
	 * @param $name, nom du fichier. (chaîne de caractères)
	 * @see getName
	 */
	function setName($name) {
		$this->name = $name;
	}

	/** 
	 * @public
	 * @return Chemin du fichier. (chaîne de caractères)
	 */
	function getPath() {
		return $this->path;
	}

	/** 
	 * @public
	 * @return Date du fichier. (time)
	 */
	function getDate() {
		return $this->date;
	}

	/** 
	 * @public
	 * @note La description peut contenir des balises HTML.
	 * @return la description du fichier au format HTML. (chaîne de caractères)
	 */
	function getDescriptionHTML(){
		return $this->description;
	}

	/** 
	 * @public
	 * @note Les caractères spéciaux seront encoder en HTML.
	 * @return la description de la photo au format texte. (chaîne de caractères)
	 */
	function getDescription(){
		return nl2br(htmlspecialchars($this->description));
	}
	//@}
	
	
	/**************************************************************************
				CONSTRUCTOR							
	**************************************************************************/
	/** @name Constructor
	 */
	//@{
	
	/** 
	 * Constructeur PHP4.
	 * @private (for the visibility in the documentation)
	 * @param $path, le chemin vers le fichier. (chaîne de caractères)
	 * @param $album, l'album parent. (objet Zphotos_Album)
	 * @param $options, les options de Zphotos Express. (objet Zphotos_Options)
	 * @see __construct()
	 */
	function Photo($path, &$album, &$options) {
		$this->__construct($path, $album, $options);
	}

	/** 
	 * Constructeur.
	 * @public
	 * @param $path, le chemin vers le fichier. (chaîne de caractères)
	 * @param $album, l'album parent. (objet Zphotos_Album)
	 * @param $options, les options de Zphotos Express. (objet Zphotos_Options)
	 */
	function __construct($path, &$album, &$options) {
		$this->path = $path;
		$this->album = &$album;
		$this->options = &$options;
		$this->name = basename($path);
		$this->relativePath = substr($path, strlen($options->getPhotosRootPath())); 
		
		// Récupération de la date
		$exploded = explode(' ', $this->name);
		if (count($exploded) > 1) {
			$date = strtotime(array_shift($exploded));
			if ($date !== -1 && $date !== false) {
				$this->date = $date;
				$this->name = join($exploded);
			}
		}
		
		// Search a description associated
		$descFileName = $this->path.'.txt';
		if (file_exists($descFileName )) {
			$this->description = implode('', file($descFileName ));
		}
	}
	//@}	
}
?>