<?php
/** @file
 *
 * @note Copyright (C) 2007-2010 Thomas ZILLIOX
 * 
 * This file is part of ZPHOTOS EXPRESS.
 * 
 * ZPHOTOS EXPRESS is free script: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License.
 * 
 * ZPHOTOS EXPRESS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with ZPHOTOS EXPRESS. If not, see <http://www.gnu.org/licenses/>.
 * 
 * You can contact the author by e-mail thomas@zilliox.me
 * 
 */

/** @class
 * @author Thomas ZILLIOX
 * @date 2007-2010
 * @version 2.0
 */
class Zphotos_View {
	
	
	/**************************************************************************
				ATTRIBUTES						
	/*************************************************************************/

	/** 
	 * Text à afficher lorsqu'un album est vide.
	 * @private
	 */
	var $emptyAlbumWording = 'Cet album ne contient aucune photo.';

	/** 
	 * Text à afficher lorsqu'un album est vide.
	 * @private
	 */
	var $credits = 'Zphotos Express V2.0';

	/** 
	 * L'album courant. (objet Zphotos_Album)
	 * @private
	 */
	var $currentAlbum;

	/** 
	 * La page courante de l'album. (nombre)
	 * @private
	 */
	var $currentPage;

	/** 
	 * La photo courante. (objet Zphotos_Photo)
	 * @private
	 */
	var $currentPhoto;


	/**************************************************************************
				PUBLIC METHODS						
	**************************************************************************/
	/** @name Public methods
	 */
	//@{

	/** 
	 * @public
	 * @param $album, L'album à afficher.
	 * @param $page, La page de l'album à afficher.
	 * @return Le code HTML de la page représentant l'album passé en paramètre.
	 */
	function showAlbum($album, $page) {
		$this->currentAlbum = $album;
		$this->currentPage = $page;
		$html = $this->_getMenuHTML($this->rootAlbum);
		$html .= $this->_getPhotosHTML($album, 0, '', true);
		
		return $html;
	}

	/** 
	 * @public
	 * @param $photo, La photo à afficher
	 * @return Le code HTML de la page représentant la photo passée en paramètre.
	 */
	function showPhoto($photo) {
		$this->currentPhoto = $photo;
		$html = $this->_getMenuHTML($this->rootAlbum);
		$html .= $this->_getPhotoHTML($photo, '', false, '', '', '');
		return $html;
	}
	//@}
	
	
	/**************************************************************************
				CONSTRUCTOR							
	**************************************************************************/
	/** @name Constructor
	 */
	//@{

	/** 
	 * Constructeur PHP4.
	 * @private (for the visibility in the documentation)
	 * @param $rootAlbum, le dossier racine des albums photos
	 * @see __construct()
	 */
	function Zphotos_View(&$rootAlbum) {
		$this->__construct($rootAlbum);
	}

	/** 
	 * Constructeur.
	 * @public
	 * @param $rootAlbum, le dossier racine des albums photos
	 */
	function __construct(&$rootAlbum) {
		$this->rootAlbum = &$rootAlbum;
	}
	//@}
	
	
	/**************************************************************************
				PRIVATE PHOTO METHODS						
	**************************************************************************/
	/** @name Private photo methods
	 */
	//@{

	
	/** 
	 * @private
	 * @param $photo, la photo à afficher. (objet Zphotos_Photo)
	 * @param $class, la classe à ajouter à la photo. (chaîne de caractères) (facultatif)
	 * @param $link, si la photo doit être dans un lien, l'url souhaité, sinon une chaîne vide. (chaîne de caractères) (facultatif, pas de lien par défaut)
	 * @param $method, méthode de redimenssionnement, valeur possible : 'crop' et 'resize'. (chaîne de caractères) (facultatif, redimmenssionnement avec déformation par défaut)
	 * @param $width, la largeur maximale de la photo en pixels. (nombre) (facultatif, 100px par défaut)
	 * @param $height, la hauteur maximale de la photo en pixels. (nombre) (facultatif, 100px par défaut)
	 * @param $title, le titre de la photo. (chaîne de caractères) (facultatif)
	 * @return la photo au format au HTML. (chaîne de caractères)
	 */
	function _getPhotoHTML($photo){
		if (func_num_args() > 1) $class = func_get_arg(1);
		else $class = '';
		if (func_num_args() > 2) $link = func_get_arg(2);
		else $link = '';
		if (func_num_args() > 3) $method = func_get_arg(3);
		else $method = '';
		if (func_num_args() > 4) $width = func_get_arg(4);
		else $width = '100';
		if (func_num_args() > 5) $height = func_get_arg(5);
		else $height = '100';
		if (func_num_args() > 6) $title = func_get_arg(6);
		else $title = '';
		
		$newWidth = $width;
		$newHeight = $height; 
		if ($method == 'resize' || $method == 'crop') {		
			// We see if the width or the height are more limited
			if ($width/$height > $photo->getWidth()/$photo->getHeight() && $method == 'resize' ||
				$width/$height < $photo->getWidth()/$photo->getHeight() && $method == 'crop') {
				$ratio = $height/$photo->getHeight();
			} else {
				$ratio = $width/$photo->getWidth();
			}
			
			// We display the photo 
			if ($method == 'resize') {
				// If the size of the Photo is smaller than the contraints
				if ($ratio <= 1) {
					$newWidth = round($photo->getWidth() * $ratio);
					$newHeight = round($photo->getHeight() * $ratio);
				}
			} elseif ($method == 'crop') {
				if ($ratio <= 1) {
					$newWidth = round($photo->getWidth() * $ratio);
					$newHeight = round($photo->getHeight() * $ratio);
				}
			}
		}
		
		$str = '';
		if ($method == 'crop') {
			$str .= '<div style="width:'.$width.'px;height:'.$height.'px;position:relative;overflow:hidden;z-index:101;"';
			if ($class !== null) $str .= ' class="'.$class.'"';
			$str .= '>';
		}
		if (!empty($link)) {
			$str .= '<a title="';
			if (!empty($title)) {
				$str .= $title;
			} elseif ($link == 'album') {
				$album = $photo->getAlbum();
				if ($album) {
						$str .= $album->getName();
				}
			} else {
				$str .= $photo->getName();
			}
			$str .= '" href="';
			if ($link == 'photo') {
				$str .= $photo->getZoomUrl();
			} elseif ($link == 'album') {
				$album = $photo->getAlbum();
				if ($album) {
					$str .= $album->getUrl();
				}
			} else {
				$str .= $link;
			}
			$str .= '">';
		}
		$str .= '<img src="'.$photo->getUrl().'"';
		if ($class !== null && $method != 'crop') $str .= ' class="'.$class.'"';
		if ($newWidth !== null) $str .=' width="'.$newWidth.'px"';
		if ($newHeight !== null) $str .=' height="'.$newHeight.'px"';
		$str .= ' />';
		if (!empty($link)) $str .= '</a>';
		if ($method == 'crop') {
			$str .= '</div>';
		}
		return $str;
	}	
	//@}
	
	
	/**************************************************************************
				PRIVATE ALBUM METHODS						
	**************************************************************************/
	/** @name Private album methods
	 */
	//@{

	/** 
	 * @private
	 * @return la classe de l'album, parmi : '', 'current', 'currentFather'. (chaîne de caractères)
	 */
	function _getClassAlbum($album) {
		if ($this->currentAlbum != null) {
			if ($album->getPath() == $this->currentAlbum->getPath()) {
				return 'current';
			} else if (zp_startswith($this->currentAlbum->getPath(), $album->getPath())) {
				return 'parent';
			}
		}
		return '';
	}

	/** 
	 * @note Méthode pratique pour créer rapidement des menus.
	 * @private
	 * @param $album, l'album racine à afficher. (objet Zphotos_Album)
	 * @param vrai si vous voulez ajouter le nombre de photo dans l'album a la suite de son nom. (booleen) (facultatif, vrai par defaut)
	 * @param vrai si vous voulez les liens vers ses sous-albums également. (booléen) (facultatif, faux par defaut)
	 * @param vrai si vous voulez le lien vers cet album. (booléen) (facultatif, vrai par defaut)
	 * @return le lien HTML vers l'album. (chaîne de caractères)
	 */
	function _getMenuHTML($rootAlbum) {
		if (func_num_args() > 1) $size = func_get_arg(1);
		else $size = true;
		if (func_num_args() > 2) $childs = func_get_arg(2);
		else $childs = true;
		if (func_num_args() > 3) $me = func_get_arg(3);
		else $me = true;
		if ($me) {
			$str = '<a href="'.$rootAlbum->getUrl().'" class="'.$this->_getClassAlbum($rootAlbum).'">'.$rootAlbum->getName();
			if ($size && !$rootAlbum->isEmpty()) $str .= ' ('.$rootAlbum->getPhotosNumber().')';
			$str .= '</a>';
		}
		if ($childs && $rootAlbum->hasAlbums()) {
			$str .= '<ul>';
			foreach($rootAlbum->getAlbums() as $album) {
				$str .= '<li>'.$this->_getMenuHTML($album, $size, true, true).'</li>';
			}
			$str .= '</ul>';
		}
		return $str;
	}
	
	/** 
	 * @private
	 * @param $album, l'album à afficher. (objet Zphotos_Album)
	 * @param la classe à ajouter à chaque photo. (chaîne de caractères) (facultatif)
	 * @param indique si un lien doit être fait vers le zoom de la photo. (booléen) (facultatif, vrai par défaut)
	 * @param méthode de redimenssionnement, valeur possible : 'crop' et 'resize'. (chaîne de caractères) (facultatif)
	 * @param la largeur maximale de chaque photo en pixels. (nombre) (facultatif, 100px par défaut)
	 * @param la hauteur maximale de chaque photo en pixels. (nombre) (facultatif, 100px par défaut)
	 * @return la liste des photos de l'album au format au HTML. (chaîne de caractères)
	 * @see getPhotoHTML()
	 */
	function _getPhotosHTML($album) {
		if (func_num_args() > 1) $page = func_get_arg(1);
		else $page;
		if (func_num_args() > 2) $class = func_get_arg(2);
		else $class = '';
		if (func_num_args() > 3) $link = func_get_arg(3);
		else $link = '';
		if (func_num_args() > 4) $method = func_get_arg(4);
		else $method = '';
		if (func_num_args() > 5) $width = func_get_arg(5);
		else $width = '100';
		if (func_num_args() > 6) $height = func_get_arg(6);
		else $height = '100';
		
		// Do the paging
		$photos = $album->getPhotos($page);
		if (count($photos) == 0) return $this->emptyAlbumWording;
		
		// Make the HTML
		$str = '<ul class="photosList">';
		foreach($photos as $photo) {
			$str.='<li>'.$this->_getPhotoHTML($photo, $class, $link, $method, $width, $height).'</li>';
		}
		return $str.'</ul>';
	}

	/** 
	 * @private
	 * @param $album, l'album à afficher. (objet Zphotos_Album)
	 * @todo ajouter les paramètres
	 * @see getPhotoHTML()
	 */
	function _getAlbumsHTML($album) {
		$str = '<ul>';
		foreach($album->getAlbums() as $tmpAlbum) {
			$str.='<li class="album">'.$this->_getPhotoHTML($tmpAlbum->getFirstPhoto(), 'diapo', 'album', 'crop', 120, 90).'
				<div class="name">'.$tmpAlbum->getName().'</div>
				</li>';
		}
		return $str.'</ul>';
	}
	
	/** 
	 * @private
	 * @note le lien vers la page actuelle a la classe "current".
	 * @param $album, l'album à afficher. (objet Zphotos_Album)
	 * @return le code HTML des liens vers les différentes pages. (chaîne de caractères)
	 * @see getPagesNumber()
	 * @see getPageUrl()
	 */
	function _getPagingMenuHTML($album) {
		$page = 1;
		$lastPage = $album->getPagesNumber();
		$str = '<span class="pagingMenu">';
		while($page <= $lastPage) {
			$class = '';
			if ($page == $album->page) $class = ' class="current"';
			$str .= '<a href="'.$album->getPageUrl($page).'"'.$class.'>'.$page.'</a></li>';
			++$page;
		}
		$str .= '</span>';
		return $str;
	}
	//@}
}
?>